/**
  *  \file client/playvcrbg.cc
  *  \brief PlayVCR Background Widget
  *
  *  (c) 2001-2015 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/playvcrbg.h"
#include "ui/uiglobal.h"
#include "ui/widgets/abstractbutton.h"
#include "gfx/palette.h"
#include "resmgr/resmgr.h"
#include "util/nls.h"
#include "sound/sound.h"
#include "client/sound-conf.h"
#include "ui/layout.h"
#include "ui/commands.h"
#include "game/pref.h"
#include "client/widgets/consoleview.h"
#include "util/console.h"
#include "ui/widgets/keytranslator.h"
#include "ui/widgets/button.h"
#include "ui/widgets/checkbox.h"
#include "ui/widgets/minimalbutton.h"

/************************** WVcrBackgroundWindow *************************/

/** Constructor. */
WVcrBackgroundWindow::WVcrBackgroundWindow()
    : WImagePanel(string_t(), requestPixmap(ResId("bg.vcr", ui_root->getExtent().w)))
{
    init();
}

/** Initialize widget. This constructs the contained buttons. */
void
WVcrBackgroundWindow::init()
{
    GfxRect r = ui_root->getExtent();
    int y = 0;
    UIWidget::LayoutInfo li;

    UIAbstractButton& b = h.add(new UIMinimalButton(0, _("F2 - Sound Config"), SDLK_F2));
    b.getLayoutInfo(li);
    b.sig_fired.add(this, &WVcrBackgroundWindow::onSoundConfig);
    b.setExtent(GfxRect(r.w - li.pref_size.x, y, li.pref_size.x, li.pref_size.y));
    addChildWidget(b, 0);
    y += li.pref_size.y + 1;

    UIAbstractButton& b2 = h.add(new UIMinimalButton(0, _("F3 - General Config"), SDLK_F3));
    b2.getLayoutInfo(li);
    b2.sig_fired.add(this, &WVcrBackgroundWindow::onGeneralConfig);
    b2.setExtent(GfxRect(r.w - li.pref_size.x, y, li.pref_size.x, li.pref_size.y));
    addChildWidget(b2, 0);
    y += li.pref_size.y + 1;

    UIAbstractButton& b3 = h.add(new UIMinimalButton(0, _("F4 - Simulator"), SDLK_F4));
    b3.getLayoutInfo(li);
    b3.sig_fired.add(this, &WVcrBackgroundWindow::onSimulate);
    b3.setExtent(GfxRect(r.w - li.pref_size.x, y, li.pref_size.x, li.pref_size.y));
    addChildWidget(b3, 0);
    y += li.pref_size.y + 1;
}

WVcrBackgroundWindow::~WVcrBackgroundWindow()
{ }

bool
WVcrBackgroundWindow::handleEvent(const UIEvent& event, bool second_pass)
{
    if (event.type == ev_Key && event.key.code == ss_Alt + 'c') {
        onConsole();
        return true;
    }
    return WImagePanel::handleEvent(event, second_pass);
}

/** "Sound config" button. Displays the sound options dialog. */
void
WVcrBackgroundWindow::onSoundConfig()
{
    SfxConfig conf = getSoundConfig();
    if (configureSoundDialog(conf)) {
        configureSound(conf);
        saveSoundConfig();
    }
}

/** "General config" button. Displays the general config dialog (i.e. 
    "Instant Battle Result" setting). */
void
WVcrBackgroundWindow::onGeneralConfig()
{
    ObjectHolder holder;
    UICheckbox cb_instant(100, _("Instant battle result"), 'i', 2);
    UIGroup h(UIHBoxLayout::instance5);

    UIWindow win(_("General Configuration"),
                 UIWindow::BLUE_WINDOW, COLOR_GRAY);
    win.setLayout(UIVBoxLayout::instance5);

    h.add(holder.add(new UISpacer()));
    h.add(holder.add(new UIButton(cm_Enter, _("OK"), SDLK_RETURN)).addStop());
    h.add(holder.add(new UIButton(cm_Escape, _("Cancel"), SDLK_ESCAPE)).addStop());

    win.add(cb_instant);
    win.add(h);
    win.pack();

    cb_instant.setValue(getUserPreferences().InstantBattleResult());
    cb_instant.setFocusedWidget();

    ui_root->centerWidget(win);
    ui_root->add(win);
    int rv = ui_root->run();
    ui_root->close(win);

    if (rv == cm_Enter) {
        getUserPreferences().InstantBattleResult.set(cb_instant.getValue());

        /* Redraw topmost item: if it is different from us, this is the VCR window. */
        if (ui_root->getFirstChild() != this)
            ui_root->getFirstChild()->drawWidget();
    }
}

/** Display console. We do not accept input, so just show the messages
    we have accumulated so far. */
void
WVcrBackgroundWindow::onConsole()
{
    // VBox
    //   WConsoleViewWithHandler
    //   HBox
    //     UISpacer
    //     UIButton "Close"

    /* Figure out minimum height */
    int size = console.getScrollbackFill();
    if (size > 15)
        size = 15;

    /* Build widgets */
    ObjectHolder h;
    UIWindow& win = h.add(new UIWindow(_("Console Messages"), UIWindow::BLUE_WINDOW, COLOR_GRAY));
    win.setLayout(UIVBoxLayout::instance5);
    win.add(h.add(new WConsoleViewWithHandler(console, size)));

    UIGroup& g = h.add(new UIGroup(UIHBoxLayout::instance5));
    g.add(h.add(new UISpacer()));
    g.add(h.add(new UIButton(0, _("Close"), SDLK_RETURN)).addStop());
    win.add(g);
    win.add(h.add(new UIKeyTranslator()).addStop(' ', 0).addStop(SDLK_ESCAPE, 0));

    win.pack();

    /* Ensure minimum width */
    int min_size    = ui_root->getExtent().w * 9 / 10;
    GfxRect win_pos = win.getExtent();
    if (win_pos.w < min_size) {
        win_pos.w = min_size;
        win.setExtent(win_pos);
    }

    /* Do it */
    ui_root->centerWidget(win);
    ui_root->runModal(win);
}
