/**
  *  \file ui/widgets/filechoose.h
  *  \brief File Chooser Widget
  *
  *  (c) 2001-2015 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_UI_WIDGETS_FILECHOOSE_H
#define PCC_V2_UI_WIDGETS_FILECHOOSE_H

#include <vector>
#include "ui/listbox.h"
#include "io/dirs.h"
#include "io/glob.h"
#include "cpluslib/sigslot.h"

/** File chooser. Displays the content of a directory and lets the
    user choose from the files in it. Content is filtered by a
    user-changeable wildcard.

    This displays a multi-column list. Directories are shown in
    boldface, files in regular font. The widget automatically chooses
    the number of columns to display based upon an estimated average
    column size, and the number of lines based upon the font height. 
    The column/lines parameters given to the constructor specify the
    preferred size.

    If the user chooses a file, signal sig_file_selected is raised. 
    The surrounding dialog interacts with this object using the
    handleUserInput and handleEnter operations. Those functions can
    also raise sig_file_selected. */
class UIFileChooser : public UIScrollable {
 public:
    UIFileChooser(int id, Ptr<Directory> initial_directory, string_t wildcard, int columns, int lines);
    ~UIFileChooser();

    void getLayoutInfo(LayoutInfo& info);
    bool handleEvent(const UIEvent& event, bool second_pass);
    void drawContent(GfxCanvas& can);
    void scrollTo(int pos, int corr);
    void onResize(GfxRect oldsize, bool by_layout);
    void onStateChange(int state, bool enable);

    void setDirectory(Ptr<Directory> directory);
    bool handleUserInput(string_t input);
    void handleEnter();

    Ptr<Directory> getCurrentDirectory() const;
    const string_t& getResultFile() const;

    /** Directory changed. */
    Signal0 sig_directory_changed;
    Signal0 sig_file_selected;

 private:
    /* Size */
    int columns;
    int lines;

    int current_columns;
    int current_column_width;

    Ptr<GfxPixmap> icons;

    /* Current directory. Null if we're looking at the root list */
    Ptr<Directory> current_directory;

    /* File name pattern */
    FileNamePattern file_pattern;

    /* Result */
    string_t result_file;

    /* Directories. Always displayed first. */
    struct Dir {
        string_t title;
        Ptr<Directory> directory;
        bool isUp;
        Dir(string_t title, Ptr<Directory> directory, bool isUp)
            : title(title), directory(directory), isUp(isUp)
            { }
    };
    std::vector<Dir> directories;

    /* Files. */
    std::vector<Ptr<DirectoryEntry> > files;

    void scanDirectory(Ptr<Directory> directory, bool interactive);
    void showDirectory(Ptr<Directory> directory);
    void updateSize();
    static bool isDirLT(const Dir& a, const Dir& b);
    static bool isEntryLT(const Ptr<DirectoryEntry>& a, const Ptr<DirectoryEntry>& b);
};

/** Get currently-displayed directory. */
inline Ptr<Directory>
UIFileChooser::getCurrentDirectory() const
{
    return current_directory;
}

/** Get resulting file. This call is valid after sig_file_selected was raised. */
inline const string_t&
UIFileChooser::getResultFile() const
{
    return result_file;
}

#endif
