/**
  *  \file except.cc
  *  \brief Exception Classes
  *
  *  We implement the "const char*", copy constructors and destructors
  *  manually to defeat compiler inlining. Without that, gcc generates
  *  about 500 bytes to 1k of code and data per throw (basically, the
  *  inlined copy constructors contain a string copy which itself
  *  might throw again, this needs a lot of code to handle the case
  *  where the exception object itself cannot be constructed(!)).
  *
  *  (c) 2001-2015 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include <cstring>
#include <cerrno>
#include "util/except.h"
#include "util/nls.h"
#include "cpluslib/format.h"
#include "cpluslib/stringcv.h"
#include "io/streams.h"

/// Construct from string.
FileProblemException::FileProblemException(const string_t& filename,
                                           const string_t& what)
    : std::runtime_error(toStandardString(what)),
      filename(filename)
{ }

/// Construct from stream.
FileProblemException::FileProblemException(Stream& s,
                                           const string_t& what)
    : std::runtime_error(toStandardString(what)),
      filename(s.getName())
{ }

/// Copy constructor.
FileProblemException::FileProblemException(const FileProblemException& e)
    : std::runtime_error(e),
      filename(e.filename)
{ }

/// Destructor.
FileProblemException::~FileProblemException() throw()
{ }

/// Construct from string.
FileOpenException::FileOpenException(const string_t& filename, const string_t& what)
    : FileProblemException(filename, what)
{ }

/// Copy constructor.
FileOpenException::FileOpenException(const FileOpenException& e)
    : FileProblemException(e)
{ }

/// Destructor.
FileOpenException::~FileOpenException() throw()
{ }


/// Construct from string.
FileFormatException::FileFormatException(const string_t& filename,
                                         const string_t& what)
    : FileProblemException(filename, what)
{ }

/// Construct from stream.
FileFormatException::FileFormatException(Stream& s, const string_t& what)
    : FileProblemException(s, what)
{ }


/// Copy constructor.
FileFormatException::FileFormatException(const FileFormatException& e)
    : FileProblemException(e)
{ }

/// Destructor.
FileFormatException::~FileFormatException() throw()
{ }


/// Construct from string.
FileTooShortException::FileTooShortException(const string_t& fname)
    : FileFormatException(fname, _("File is too short"))
{ }

/// Construct from stream.
FileTooShortException::FileTooShortException(Stream& s)
    : FileFormatException(s.getName(), _("File is too short"))
{ }

/// Construct from C-string.
FileTooShortException::FileTooShortException(const char* fname)
    : FileFormatException(fname, _("File is too short"))
{ }

/// Copy constructor.
FileTooShortException::FileTooShortException(const FileTooShortException& e)
    : FileFormatException(e)
{ }

/// Destructor.
FileTooShortException::~FileTooShortException() throw()
{ }


/// Construct from string.
FileIOException::FileIOException(const string_t& filename, const string_t& what)
    : FileProblemException(filename, what)
{ }

/// Construct from stream.
FileIOException::FileIOException(Stream& s, const string_t& what)
    : FileProblemException(s, what)
{ }

/// Copy constructor.
FileIOException::FileIOException(const FileIOException& e)
    : FileProblemException(e)
{ }

/// Destructor.
FileIOException::~FileIOException() throw()
{ }


/// Construct from filename.
FileWriteException::FileWriteException(const string_t& filename)
    : FileIOException(filename, _("Write error"))
{ }

/// Construct from stream.
FileWriteException::FileWriteException(Stream& s)
    : FileIOException(s.getName(), _("Write error"))
{ }

/// Copy constructor.
FileWriteException::FileWriteException(const FileWriteException& e)
    : FileIOException(e)
{ }

/// Destructor.
FileWriteException::~FileWriteException() throw()
{ }


/// Construct from string.
GraphicsIOException::GraphicsIOException(const string_t& awhat)
    : std::runtime_error(toStandardString(awhat))
{ }

/// Construct from C-string.
GraphicsIOException::GraphicsIOException(const char* awhat)
    : std::runtime_error(awhat)
{ }

/// Copy constructor.
GraphicsIOException::GraphicsIOException(const GraphicsIOException& e)
    : std::runtime_error(e)
{ }

/// Destructor.
GraphicsIOException::~GraphicsIOException() throw()
{ }


/// Construct from string.
UserException::UserException(const string_t& awhat)
    : std::runtime_error(toStandardString(awhat))
{ }

/// Construct from C-string.
UserException::UserException(const char* awhat)
    : std::runtime_error(awhat)
{ }

/// Copy constructor.
UserException::UserException(const UserException& e)
    : std::runtime_error(e)
{ }

/// Destructor.
UserException::~UserException() throw()
{ }


/// Construct from string.
InitException::InitException(const string_t& awhat)
    : std::runtime_error(toStandardString(awhat))
{ }

/// Construct from C-string.
InitException::InitException(const char* awhat)
    : std::runtime_error(awhat)
{ }

/// Copy constructor.
InitException::InitException(const InitException& e)
    : std::runtime_error(e)
{ }

/// Destructor.
InitException::~InitException() throw()
{ }
